# schoolsyst: defining possible user actions from data

This will hopefully help define a clear set of components.

## Accounts

Accounts (pydantic):

```python
class User(BaseModel):
    key: UserKey = Field(default_factory=userkey, alias="_key")
    joined_at: datetime
    username: UsernameStr  # unique
    email: EmailStr  # unique
    email_is_confirmed: bool = False

class DBUser(User):
    password_hash: str


class InUser(BaseModel):
    username: UsernameStr
    email: EmailStr
    password: str
```

### View data

- username
- email
- email_is_confirmed

### Create data

- create an account:
  - account creation form
    - password field
    - text field
      - with validatiors:
        - email
        - username

### Modify data

- change email
- confirm email
- change password

### Delete data

- delete account

## Grades

```python
class Grade(OwnedResource):
    title: str
    actual: Optional[Primantissa] = None
    expected: Optional[Primantissa] = None
    goal: Optional[Primantissa] = None
    unit: confloat(gt=0)
    weight: PositiveFloat = 1
    obtained_at: Optional[datetime] = None
```

### View data

- title
- actual, expected, goal
- unit
- weight

### Create data

- add grades
- add actual
- add expected
- add goal

### Modify data

- change grade title
- change actual, expected, goal
- change unit
- change weight

### Delete data

- delete grade
- delete actual
- delete expected
- delete goal

## Homework

```python
class HomeworkType(StrEnum):
    test = auto()
    coursework = auto()
    to_bring = auto()
    exercise = auto()


class Task(BaseModel):
    key: str = Field(default_factory=objectbarekey)
    title: str
    completed: bool = False
    completed_at: Optional[datetime] = None


class InHomework(BaseModel):
    title: str
    subject_key: Optional[ObjectKey]
    type: HomeworkType
    details: str = ""
    completed_at: Optional[datetime] = None
    due_at: Optional[datetime] = None
    progress: Primantissa = 0
    tasks: List[Task] = []
    notes: List[ObjectKey] = []
    grades: List[ObjectKey] = []

    @property
    def completed(self) -> bool:
        return self.progress >= 1
```

### View data

- title
- subject
- type
- details
- due_at
- progress (and/or completed)
- tasks
- notes (-> title)
- grades (-> title)

### Add data

- add homework
- add tasks to homework
- add link to grade
- add link to note
- add a subject
- add a due date

### Modify data

- change title
- complete homework
- complete task
- re-order tasks
- change homework type
- change due date
- change details
- change task title
- change subject

### Delete data

- delete link to grade
- delete link to note
- delete task
- delete due date
- remove subject

## Schedule

```python
class InEvent(BaseModel):
    subject_key: ObjectKey
    start: time
    end: time
    day: ISOWeekDay
    room: str = ""
    on_even_weeks: bool = True
    on_odd_weeks: bool = True


class EventMutationInterpretation(StrEnum):
    edit = auto()
    reschedule = auto()
    addition = auto()
    deletion = auto()


class EventMutation(OwnedResource):
    event_key: Optional[ObjectKey] = None
    subject_key: Optional[ObjectKey] = None
    deleted_in: Optional[DatetimeRange] = None
    added_in: Optional[DatetimeRange] = None
    room: Optional[str] = None

    @property
    def interpretation(self) -> Optional[EventMutationInterpretation]:
        ...


class Course(OwnedResource):
    subject_key: ObjectKey
    start: datetime
    end: datetime
    room: str
```

### View data

- view _base_ events
  
  - subject
  
  - start
  
  - end
  
  - day
  
  - room
  
  - on_{even,odd}_weeks

- view course for a time period
  
  - view time period
  
  - view courses
    
    - subject
    
    - start
    
    - end
    
    - day
    
    - room

- view mutations
  
  - edit
    
    - subject (if changed)
      
      - title
      
      - color
    
    - room (if changed)
  
  - reschedule
    
    - subject
      
      - title/color
    
    - initial
      
      - day/start/end
    
    - modified
      
      - day/start/end
  
  - deletion or deletion
    
    - subject/room
    
    - day/start/end

### 


